@@ -9,16 +9,17 @@ package List::Gen::Perl6;
         FILTER_ONLY      all => \&_filter_hyper,
             code_no_comments => \&_filter_rest;
     }
-    my $ops = join '|' => map quotemeta, ',', qw (+ - / * ** x % . & | ^ < > <<
-                                      >> <=> cmp lt gt eq ne le ge == != <= >=);
+    my ($ops) = map qr/(?:[Rr]?(?:$_))/, join '|', map quotemeta, ',', qw (
+        - + / * ** x % . & | ^ < > << >> <=> cmp lt gt eq ne le ge == != <= >=
+    );
     sub _filter_hyper {
-        s/ (<<|>>) (R?(?:$ops)) (<<|>>) /$1'$2'$3/gx;
+        s/ ((?:<<|>>)~?) ($ops) (<<|>>) /$1'$2'$3/gx;
     }
     sub _filter_rest {
         s{
             (?<!<)
             \[ (\.\.|\\)? ($ops) \]
-            (?= \s* (?! \b$ops\b | [;\)\]\}\>] ) )
+            (?= \s* (?! -> | \b(?:$ops)\b(?!>) | [;\)\]\}\>] ) )
         }{
             my $t = $1 || '';
             $t = '..' if $t eq '\\';
@@ -34,14 +35,16 @@ package List::Gen::Perl6;
             "List::Gen::Perl6::_reduction('$t$2')"
         }gxe;
         s{
-            (?<![\$\@\%\&\*]) \b Z \b ($ops)?
+            (?<![\$\@\%\&\*]) \b Z([Rr]?) \b (?:(~?)($ops))?
         }{
-            $1 ? "|'$1'|" : '|'
+            my $rev = $1||$2 ? '~' : '';
+            $3 ? "|$rev'$3'|" : '|'
         }gxe;
         s{
-            (?<![\$\@\%\&\*]) \b X \b ($ops)?
+            (?<![\$\@\%\&\*]) \b X([Rr]?) \b (?:(~?)($ops))?
         }{
-            $1 ? "x'$1'x" : 'x'
+            my $rev = $1||$2 ? '~' : '';
+            $3 ? "x$rev'$3'x" : 'x'
         }gxe;
     }
 
@@ -72,17 +75,17 @@ List::Gen::Perl6 - perl6 meta operators in perl5
 
 =head1 SYNOPSIS
 
-Many of the features found in L<List::Gen> borrow ideas from perl6.  However,
+many of the features found in L<List::Gen> borrow ideas from perl6.  however,
 since the syntax of perl5 and perl6 differ, some of the constructs in perl5 are
 longer/messier than in perl6.  C< List::Gen::Perl6 > is a source filter that
 makes some of C<List::Gen>'s features more syntactic.
 
 the new syntactic constructs are:
 
-    cross:     generator X  generator
-    crosswith: generator X+ generator
     zip:       generator Z  generator
     zipwith:   generator Z+ generator
+    cross:     generator X  generator
+    crosswith: generator X+ generator
     hyper:     generator <<+>> generator
     hyper:     generator >>+<< generator
     hyper:     generator >>+>> generator
@@ -100,11 +103,13 @@ the native overloaded ops, and the operation expanded into methods and functions
 
     <1..3> Z <4..6>      ~~  <1..3> | <4..6>        ~~  <1..3>->zip(<4..6>)
 
-    <1..3> Z. <4..6>     ~~  <1..3> |'.'| <4..6>    ~~  <1..3>->zipwith(sub {$_[0] . $_[1]}, <4..6>)
+    <1..3> Z. <4..6>     ~~  <1..3> |'.'| <4..6>    ~~  <1..3>->zip('.' => <4..6>)
 
     <1..3> X <4..6>      ~~  <1..3> x <4..6>        ~~  <1..3>->cross(<4..6>)
 
-    <1..3> X. <4..6>     ~~  <1..3> x'.'x <4..6>    ~~  <1..3>->crosswith(sub {$_[0] . $_[1]}, <4..6>)
+    <1..3> X. <4..6>     ~~  <1..3> x'.'x <4..6>    ~~  <1..3>->cross('.' => <4..6>)
+
+    <1..3> <<+>> <4..6>  ~~  <1..3> <<'+'>> <4..6>  ~~  <1..3>->hyper('<<+>>', <4..6>)
 
     [+] 1..10            ~~  <[+] 1..10>            ~~  reduce {$_[0] + $_[1]} 1 .. 10
     [+]->(1..10)         ~~  <[+]>->(1..10)         ~~  same as above
@@ -112,28 +117,36 @@ the native overloaded ops, and the operation expanded into methods and functions
     [\+] 1..10           ~~  <[..+] 1..10>          ~~  scan {$_[0] + $_[1]} 1 .. 10
     [\+]->(1..10)        ~~  <[..+]>->(1..10)       ~~  same as above
 
-    <1..3> <<+>> <4..6>  ~~  <1..3> <<'+'>> <4..6>  ~~  gen {$$_[0] + $$_[1]} tuples <1..3>, <4..6>
+except for normal reductions C< [+] >, all of the new constructs return a
+generator.
 
-Except for normal reductions C< [+] >, all of the new constructs return a generator.
+you can flip the arguments to an operator with C< R > or C< r > and in some
+cases C< ~ >
 
-When used without a following argument, reductions and triangular reductions will return
-a code reference that will perform the reduction on its arguments.
+      ZR.     Zr.     Z~.
+      XR.     Xr.     X~.
+     <<R.>>  <<r.>>  <<~.>>
+      [R.]    [r.]    n/a
+     [\R.]   [\r.]    n/a
+
+when used without a following argument, reductions and triangular reductions
+will return a code reference that will perform the reduction on its arguments.
 
     my $sum = [+];
     say $sum->(1..10);  # 55
 
-Reductions can take a list of scalars, or a single generator as their argument.
+reductions can take a list of scalars, or a single generator as their argument.
 
-Only the left hand side of the zip, cross, and hyper operators needs to be a
+only the left hand side of the zip, cross, and hyper operators needs to be a
 generator.  zip and cross will upgrade their rhs to a generator if it is an array.
 hyper will upgrade it's rhs to a generator if it is an array or a scalar.
 
-The source filter is limited in scope, and should not harm other parts of the code,
+the source filter is limited in scope, and should not harm other parts of the code,
 however, source filters are notoriously difficult to fully test, so take that
-with a grain of salt.  Due to limitations of L<Filter::Simple>, hyper operators
-will be filtered in both code and strings.  All other filters should skip strings.
+with a grain of salt.  due to limitations of L<Filter::Simple>, hyper operators
+will be filtered in both code and strings.  all other filters should skip strings.
 
-This code is not really intended for serious work, ymmv.
+this code is not really intended for serious work, ymmv.
 
 =head1 AUTHOR
 
@@ -158,4 +171,4 @@ see http://dev.perl.org/licenses/ for more information.
 
 =cut
 
-1;
+1
